/* $Id: module.h,v 1.10 2000/05/26 23:04:23 becka Exp $
******************************************************************************

   KGIcon - kernel module internal interface definitions

   Copyright (C) 1997	Andreas Beck	[becka@ggi-project.org]
   Copyright (C) 1997	Steffen Seeger	[seeger@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _KGI_MODULE_H
#define _KGI_MODULE_H

#include <kgi/kgi.h>
#include <kgi/param.h>

/* !!!	These make up the new internal module interface.
*/

extern void DAC_out8(struct kgi_display *dpy, kgiu8 val, kgiu8 reg);
extern kgiu8 DAC_in8(struct kgi_display *dpy, kgiu8 reg);
extern void DAC_outs8(struct kgi_display *dpy, kgiu8 reg, void *buf, kgi_uint cnt);
extern void DAC_ins8(struct kgi_display *dpy, kgiu8 reg, void *buf, kgi_uint cnt);

extern void CLK_out8(struct kgi_display *dpy, kgiu8 val, kgiu8 reg);
extern kgiu8 CLK_in8(struct kgi_display *dpy, kgiu8 reg);

#define	COMMAND_ARGS struct kgi_graphic*, kgi_uint cmd, kgi_uint arg
#define	CHECK_ARGS struct kgi_display *, struct kgi_mode *, enum kgi_tm_cmd *
#define SET_ARGS   struct kgi_display *, struct kgi_mode *

extern int	kgim_monitor_init (struct kgi_card *, struct kgi_display *);
extern void	kgim_monitor_done (struct kgi_display *dpy);
extern void	kgim_monitor_reset (void);
extern int	kgim_monitor_check_mode(CHECK_ARGS);
extern void	kgim_monitor_set_mode (SET_ARGS);
extern int	kgim_monitor_command (COMMAND_ARGS);

extern int	kgim_ramdac_init (struct kgi_card *, struct kgi_display *);
extern void	kgim_ramdac_done (struct kgi_display *dpy);
extern void	kgim_ramdac_reset (void);
extern int	kgim_ramdac_check_mode (CHECK_ARGS);
extern void	kgim_ramdac_set_mode (SET_ARGS);
extern int	kgim_ramdac_command (COMMAND_ARGS);

extern int	kgim_accel_init (struct kgi_card *, struct kgi_display *);
extern void	kgim_accel_done (struct kgi_display *dpy);
extern void	kgim_accel_reset (void);
extern int	kgim_accel_check_mode (CHECK_ARGS);
extern void	kgim_accel_set_mode (SET_ARGS);
extern int	kgim_accel_command (COMMAND_ARGS);

extern int	kgim_vidio_init (struct kgi_card *, struct kgi_display *);
extern void	kgim_vidio_done (struct kgi_display *dpy);
extern void	kgim_vidio_reset (void);
extern int	kgim_vidio_check_mode (CHECK_ARGS);
extern void	kgim_vidio_set_mode (SET_ARGS);
extern int	kgim_vidio_command (COMMAND_ARGS);

extern int	kgim_clock_init (struct kgi_card *, struct kgi_display *);
extern void	kgim_clock_done (struct kgi_display *dpy);
extern void	kgim_clock_reset (void);
extern int	kgim_clock_check_mode (CHECK_ARGS);
extern void	kgim_clock_set_mode (SET_ARGS);
extern int	kgim_clock_command (COMMAND_ARGS);

extern int	kgim_chipset_init (struct kgi_card *, struct kgi_display *);
extern void	kgim_chipset_done (struct kgi_display *dpy);
extern void	kgim_chipset_reset (void);
extern int	kgim_chipset_check_mode (CHECK_ARGS);
extern void	kgim_chipset_set_mode (SET_ARGS);
extern int	kgim_chipset_command (COMMAND_ARGS);

#undef	COMMAND_ARGS
#undef	CHECK_ARGS
#undef	SET_ARGS

struct kgim_private
{
	void	*driver;
	void	*monitor;
	void	*chipset;
	void	*ramdac;
	void	*clock;
	void	*accel;
};

#define	KGIM_PRIVATE(dpy, subsystem) \
	(((struct kgim_private *) dpy->driver_data)->subsystem)

struct kgim_options_pci 
{
	pcicfg_vaddr	dev;
	mem_addr	base0;
	mem_addr	base1;
	mem_addr	base2;
	mem_addr	base3;
	mem_addr	base4;
};

struct kgim_options_misc
{
	mem_addr	law_base;
	kgi_uint	display;
};

struct kgim_options_chipset
{
	const char	*chipset;
};

struct kgim_options
{
	struct kgim_options_misc	*misc;
	struct kgim_options_pci		*pci;
	struct kgim_options_chipset	*chipset;
};

#define	KGIM_OPTIONS(dpy, opt)	\
	(((struct kgim_options *) KGIM_PRIVATE(dpy, driver))->opt)

#endif	/* _KGI_MODULE_H */
